# -*- coding: utf-8 -*-
import ldap, json, copy, uuid, decimal, datetime, ldap.modlist
from adobjstype import *

class adObj(object):
	'''Class to represent a LDAP Object using python object attributes like LDAP attributes
	and some methods to manage data on LDAP protocol

		Example:
			obj.dn -> Object LDAP DN attribute
			obj.samaccountname -> Object LDAP sAMAccountName attribute
			obj.description -> Object LDAP description attribute
	'''

	objtype = {}
	newobj = []

	@classmethod
	def is_type(self,data):
		'''This method checks if the data coincide with the object ldap

			@params:
				(dict)data: Dictionary object representing an LDAP object

			@return: (bool) if data is type then return True, else return False
		'''
		for i in self.objtype:
			dt = data.copy().get(i,None)
			obj = self.objtype.copy().get(i)
			if dt != None:
				dt.sort()
				obj.sort()
				if dt == obj:
					return True
		return False
		
	def __init__(self,adldap,dn,data):
		'''Constructor method

			@params:
				(adLDAP)adldap: Instance adLDAP object
				(str)dn: The DN ldap object
				(data)data: LDAP Object attributes dictionary

			@attributes:
				(bool)is_new : set True if the adObj is new to create LDAP object
				(bool)adcomplete: set True if initial object loading is complete
				(adLDAP)adldap : set instance adLDAP object

			@return: None
		'''
		self.is_new = False
		self.adcomplete = False
		self.adldap = adldap
		if dn == None: #NewObj
			for attr in self.newobj:
				setattr(self,attr.lower(),None)

		setattr(self,"dn",dn)
		for i in data:
			setattr(self,i.lower(),data[i])
		self.adcomplete = True

	def __repr__(self):
		'''Special method to return repr data
		'''
		return repr(self.dn)

	def __getitem__(self,item):
		'''Special method to get attributes to object like dictionary
			
			@params:
				(string)item : LDAP attribute object representation

			@return: (ADGenericAttr) Return AD Object type LDAP Attribute
		'''
		return self.__dict__[item]

	def __setitem__(self,field,value):
		'''Special method to get attributes to object like dictionary

			@params:
				(string)item : LDAP attribute object representation
				(anyone)value : LDAP value to set attribute object

			@return: None
      '''
	
		setattr(self,field,value)

	def __iter__(self):
		dct = self.__dict__.copy()
		del dct['adcomplete']
		del dct['adldap']
		del dct['is_new']
		return iter(dct)

	def __str__(self):
		'''Special method to print human representation oject

			Example:
				print obj

		'''
		data = self.__dict__.copy()
		del data['adldap']
		del data['adcomplete']
		del data['is_new']
		result = {}
		for i in data:
			result[i] = data[i].humanReadeable()
		return json.dumps(result,ensure_ascii=False, indent=3)

	def __setattr__(self,field,value):
		'''Special method to set Object with Object Types Attributes
		
			@params:
				(string)field : LDAP attribute name object representation
				(anyone)value : set LDAP attribute object value

			@return: None
		'''

		if (field == "adldap") or (field == 'adcomplete') or (field == 'is_new'):
			super(adObj,self).__setattr__(field,value)
			return

		if value == None:
			value = []

		if type(value) != list:
			if str(value).strip() == '':
				value = []
			else:
				value = [str(value)]

		adtypes = {
			'dn': dn,
			'msmqsigncertificates' : mSMQSignCertificates,
			'useraccountcontrol' : UserAccountControl,
			'lastlogon' : lastLogon,
			'pwdlastset' : pwdLastSet,
			'lastlogontimestamp' : lastLogonTimeStamp,
			'accountexpires' : accountExpires,
			'badpasswordtime' : badPasswordTime,
			'dscorepropagationdata' : dSCorePropagationData,
			'whenchanged' : WhenChanged,
			'whencreated' : WhenCreated,
			'objectguid' : ObjectGUID,
			'objectsid' : ObjectSid,
			'sidhistory' : sIDHistory,
			'samaccountname' : sAMAccountName,
			'memberof' : memberOf,
			'member' : member,
			'adgenericattr' : adGenericAttr,
			'logonhours' : logonHours,
			'userworkstations' : userWorkStations,
			'manager' : manager,
			'directreports' : directreports,
			'unicodePwd' : unicodePwd,
			'primarygroupid' : PrimaryGroupID,
		}

		try:
			val = adtypes[field](self.adldap,value)
		except:
			val = adtypes['adgenericattr'](self.adldap,value)

		if self.adcomplete:
			val.is_modified = True

		super(adObj,self).__setattr__(field,val)

	@property
	def is_enable(self):
		'''Method to check if LDAP Object is enable (True) or disable (False)
		'''
		if type(self) != adUser:
			raise self.adldap.ObjectNotHaveEnable("Objet not have enable attribute")
		mod = int(self.useraccountcontrol.value) % 8;
		if mod == 0:
			return True
		else:
			return False

	def delete(self):
		'''Method to delete LDAP Object from Active Directory
		'''
		self.adldap.ldapConnection.delete_s(self.dn.value)

	def move(self,newcontainer):
		'''Method to move LDAP object to another container like OU
		
			@params:
				(string)newcontainer : New Active Directory DN cotainer

			@return: None

			Example:
				obj.move("OU=OUNAME,DC=DCPREFIX,DC=DCSUFFIX")
		'''
		self.adldap.ldapConnection.rename_s(self.dn.value,'cn='+self.cn.value, newcontainer)

	def save(self):
		'''Method to create or modify the LDAP object in Active Directory
		
			If adOBJ is a New object then create LDAP object in Active Directory
			If adOBJ is not a new object then modify LDAP object in Active Directory
		'''
		modlist = []
		attrs = self.__dict__.copy()
		del attrs['adldap']
		del attrs['adcomplete']
		isNew = attrs.get('is_new',False)
		try:
			del attrs['is_new']
		except:
			pass

		if attrs.get('dn',None).value == None:
			raise self.adldap.NotDNDefinied("Not DN Definied")

		if not isNew: #is modify action
			if attrs['cn'].__dict__.get('is_modified',False):
				cn = attrs['cn'].value
				del attrs['cn']
				self.adldap.ldapConnection.modrdn_s(self.dn.value,'cn='+str(cn),True)
			for attr in attrs:
				modtype= ldap.MOD_REPLACE
				if attrs[attr].__dict__.get('is_modified',False):
					if attrs.get(attr,None) == None:
						modtype = ldap.MOD_DELETE
					modlist.append((modtype,attr,attrs[attr].raw))
					if (attr.lower() == 'unicodepwd'): #Two times is needed for update password in AD
						modlist.append((modtype,attr,attrs[attr].raw))
			if len(modlist) > 0:
				self.adldap.ldapConnection.modify_s(self.dn.value,modlist)	

		else:
			dn = attrs['dn'].value
			del attrs['dn']
			add = {}
			
			try:
				if (self.is_enable) and (self.__dict__.get("unicodePwd",None) == None):
					raise Exception("Not set Password to new enable object")
			except self.adldap.ObjectNotHaveEnable:
				pass

			for attr in attrs:
				if (attr in self.newobj) and (attrs[attr].value == None):
					raise self.adldap.EmptyAttrNewObj("Empty attributes for new Object "+str(self.newobj))
				add[attr] = attrs[attr].value
			addlist= ldap.modlist.addModlist(add)
			self.adldap.ldapConnection.add_s(dn,addlist)


class adUser(adObj):
	'''
		adObj child object class to represent LDAP Users
	'''
	objtype = {
		'objectClass' : ['top','person', 'organizationalPerson', 'user']
	}
	newobj = [
		'objectclass',
		'useraccountcontrol',
		'userprincipalname',
		'samaccountname',
		'cn',
		'givenname',
		'sn',
		'displayname',
	]

	def disable(self):
		'''Method to disable User in Active Directory'''
		self.useraccountcontrol = ["NORMAL_ACCOUNT","ACCOUNTDISABLE"]
		self.save()

	def enable(self):
		'''Method to enable User in Active Directory'''
		self.useraccountcontrol = ["NORMAL_ACCOUNT"]
		self.save()
	
	def setPassword(self,password):
		'''Set User password'''
		self.unicodePwd = password
		self.save()

	def PasswordExpiry(self):
		'''Get when the user password is expired'''
		pwdLastSet = self.pwdlastset.raw[0]

		if self.useraccountcontrol == '66048':
			return "Does not expire"

		if (pwdLastSet == 0):
			return "Password has expired"
		try:
			maxPwdAge = self.adldap.getADmaxPwdAge()
			mod = int(maxPwdAge) % 4294967296
		except:
			mod = 0
		if mod == 0:
			return "Domain does not expire passwords"

		pwdExpire = decimal.Decimal(pwdLastSet) - decimal.Decimal(maxPwdAge)

		expiryts = int((pwdExpire / 10000000) - 11644473600)

		return datetime.datetime.fromtimestamp(expiryts)


class adGroup(adObj):
	'''
		adObj child object class to represent LDAP Groups
	'''
	objtype = {
		'objectClass' : ['top', 'group']
	}
	newobj = [
		'samaccountname',
		'objectclass',
		'cn',
	]

	def AddToGroup(self,obj):
		'''
			Method to add LDAP object to Group

			@params:
				(adOBJ|DN string)obj: adOBJ or Child or LDAP DN Object string

			@return: None
		'''
		if type(obj) != str:
			dn = str(obj.dn)
		else:
			dn = obj

		try:
			self.member.raw.index(dn)
			modify = False
		except:
			modify = True

		if modify:
			lst = self.member.raw
			lst.append(dn)
			self.member = lst
			self.save()

	def DelFromGroup(self,obj):
		'''
			Method to add LDAP object to Group

			@params:
				(adOBJ|DN string)obj: adOBJ or Child or LDAP DN Object string

			@return: None
		'''
		if type(obj) != str:
			dn = str(obj.dn)
		else:
			dn = obj

		try:
			pos = self.member.raw.index(dn)
			modify = True
		except:
			modify = False

		if modify:
			lst = self.member.raw
			del lst[pos]
			self.member = lst
			self.save()


class adComputer(adObj):
	'''
		adObj child object class to represent LDAP Computers
	'''	
	objtype = {
		'objectClass' : ['top','person', 'organizationalPerson', 'user', 'computer']
	}
	newobj = [
		'objectclass',
		'cn',
		'samaccountname',
		'useraccountcontrol',
	]

	def disable(self):
		self.useraccountcontrol = ["ACCOUNTDISABLE","PASSWD_NOTREQD","WORKSTATION_TRUST_ACCOUNT"]
		self.save()

	def enable(self):
		self.useraccountcontrol = ["PASSWD_NOTREQD","WORKSTATION_TRUST_ACCOUNT"]
		self.save()

class adOU(adObj):
	'''
		adObj child object class to represent LDAP OUs
	'''
	objtype = {
		'objectClass' : ['top', 'organizationalUnit']
	}
	newobj = [
		'ou',
		'objectclass',
	]


class adObjs(object):
	'''Class to create or get adOBjs and childs objects'''
	qryFilter = ''
	adObjType = adObj

	def __init__(self,adldap):
		'''Constructor method'''
		self.adldap = adldap

	def filter(self,query,attr=['*'],ldapscope=ldap.SCOPE_SUBTREE,control_type=True,baseDN=None):
		'''Get multiple adObjs and Childs objects from query ldap sententes

			@params:
				(string)query : LDAP query sentence format
				(list)attr : Attributes to get from LDAP. Default all attributes.
				(ldap.SCOPE_X)ldapscope : Set ldap SCOPE from pytho ldap class attribute
				(bool)control_type : set True if return objects must be especific adObj child or simple adObj object
				(string)baseDN : set baseDN string

			@return: adObj and/or Childs list objects
		'''
		if baseDN == None:
			baseDN = self.adldap.baseDN

		COOKIE = ''
		CRITICALY = True
		PAGE_SIZE=1000
		result = []
		first_pass = True
		pg_ctrl = ldap.controls.SimplePagedResultsControl(CRITICALY,PAGE_SIZE,COOKIE)

		if control_type:
			query = '(&%(filter)s%(query)s)' % {'filter': self.qryFilter, 'query': query}
		else:
			query = '(&%(query)s)' % {'query': query}

		while first_pass or pg_ctrl.cookie:
			first_pass = False
			msgid = self.adldap.ldapConnection.search_ext(baseDN,ldapscope,query,attr,serverctrls=[pg_ctrl])
			result_type, data, msgid, serverctrls = self.adldap.ldapConnection.result3(msgid)
			pg_ctrl.cookie = serverctrls[0].cookie
			for obj in data:
				if obj[0] == None:
					continue
				if adUser.is_type(obj[1]):
					result.append(adUser(self.adldap,obj[0],obj[1]))
					continue
				if adComputer.is_type(obj[1]):
					result.append(adComputer(self.adldap,obj[0],obj[1]))
					continue
				if adGroup.is_type(obj[1]):
					result.append(adGroup(self.adldap,obj[0],obj[1]))
					continue
				if adOU.is_type(obj[1]):
					result.append(adOU(self.adldap,obj[0],obj[1]))
					continue
				result.append(adObj(self.adldap,obj[0],obj[1]))

		return result

	def new(self):
		'''Method to create new adObj'''
		obj = self.adObjType(self.adldap,None,self.adObjType.objtype)
		obj.is_new = True

		return obj


	def get(self,query,attr=['*'],ldapscope=ldap.SCOPE_SUBTREE,control_type=True):
		'''Filter method Wrapper to return one result adObj or Child object'''
		result = self.filter(query,attr,ldapscope,control_type)
		if len(result) == 1:
			return result[0]
		elif len(result) > 1:
			raise self.adldap.MultipleResults("Multiple Results")
		else:
			raise self.adldap.ObjectNotExist("Object not exist")

class adGroups(adObjs):
	'''Child object from adObjs to manage only Groups LDAP Objects'''
	qryFilter = '(&(objectclass=top)(objectclass=group))'
	adObjType = adGroup


class adComputers(adObjs):
	'''Child object from adObjs to manage only Gomputers LDAP Objects'''
	qryFilter = '(&(objectclass=top)(objectclass=person)(objectclass=organizationalPerson)(objectclass=user)(objectclass=computer))'
	adObjType = adComputer


class adUsers(adObjs):
	'''Child object from adObjs to manage only Users LDAP Objects'''
	qryFilter = '(&(objectclass=top)(objectclass=person)(objectclass=organizationalPerson)(objectclass=user)(!(objectclass=computer)))'
	adObjType = adUser

class adOUs(adObjs):
	'''Child object from adObjs to manage only OUs LDAP Objects'''
	qryFilter = '(&(objectclass=top)(objectclass=organizationalUnit))'
	adObjType = adOU



# -*- coding: utf-8 -*-

class adUtils(object):
	'''Class with useful tools to facilitate using the framework 
	'''
	class ObjectNotHaveEnable(Exception):
		pass

	class MultipleResults(Exception):
		pass

	class ObjectNotExist(Exception):
		pass

	class NotDNDefinied(Exception):
		pass

	class EmptyAttrNewObj(Exception):
		pass

	class NotUACValueSelected(Exception):
		pass

	acControl = {
	'''text'''
		"SCRIPT" : 1,
		"ACCOUNTDISABLE" : 2,
		"HOMEDIR_REQUIRED" : 8,
		"LOCKOUT" : 16,
		"PASSWD_NOTREQD" : 32,
		"ENCRYPTED_TEXT_PWD_ALLOWED" : 128,
		"TEMP_DUPLICATE_ACCOUNT" : 256,
		"NORMAL_ACCOUNT" : 512,
		"INTERDOMAIN_TRUST_ACCOUNT" : 2048,
		"WORKSTATION_TRUST_ACCOUNT" : 4096,
		"SERVER_TRUST_ACCOUNT" : 8192,
		"DONT_EXPIRE_PASSWORD" : 65536,
		"MNS_LOGON_ACCOUNT" : 131072,
		"SMARTCARD_REQUIRED" : 262144,
		"TRUSTED_FOR_DELEGATION" : 524288,
		"NOT_DELEGATED" : 1048576,
		"USE_DES_KEY_ONLY" : 2097152,
		"DONT_REQ_PREAUTH" : 4194304,
		"PASSWORD_EXPIRED" : 8388608,
		"TRUSTED_TO_AUTH_FOR_DELEGATION" : 16777216
	}


	@classmethod
	def littleEndian(self,hex):
		'''Method like class to calculate the littleEndian from hexadecimal value
			@param:
				(string)hex : Hexadecimal value
			@return: (string)Little Endian value 
		'''
		result = '';
		xinit = len(hex) - 2
		for x in range(xinit,0-1,-2):
			result += hex[x:x+2]
		return result


	@classmethod
	def getTextSID(self,binsid):
		'''Method like class to get a SID value from binsid
			@param:
				(string)binsid: binary sid data
			@return: (string) Text SID
		'''
		hex_sid = binsid.encode("hex")
		rev = int(hex_sid[0:0+2],16)
		subcount = int(hex_sid[2:2+2],16)
		auth = int(hex_sid[4:4+12],16)
		result = str(rev)+'-'+str(auth)

		subauth = {}

		for x in range(0,subcount):
			le = hex_sid[16+(x*8):(16+(x*8))+8]
			subauth[x] = int(self.littleEndian(le),16)
			result += '-'+str(subauth[x])

		return 'S-'+result

	def accountControl(self,options):
		'''Method to calculate useraccountcontrol attribute value
			
			@params:
				(list)options: Options values list from acControl attribute keys

			@return: (int) useraccountconltrol value
		'''
		val= 0;
		for i in options:
			val = val + self.acControl.get(i,0)
		return val

	def setQuery(self,operator='&',**fields):
		'''Method to generate ldap query sentences from **fields param dict
			
			This method generate "AND" global sentence but also can generate NOT, LESS That, MORE That, and Equal subsentences using follow suffix:
				
				attrubute="value" -> (attribute=value)
				attribute__not="value" -> (!(attribute=value))
				attribute__gt="value" -> (attribute>value)
				attribute__lt="value" -> (attribute<value)
				attribute__gt__not="value" -> (!(attribute>value))

			If value is equal to LIST then method generate multiple subsentences:

				attribute=["value1","value2","value3"] -> (attribute=value1)(attribute=value2)(attribute=value3)
				attribute__not=["value1","value2","value3"] -> (!(attribute=value1))(!(attribute=value2))(!(attribute=value3))

			@params:
				(char)operator: set the operator global sentence. & -> AND, | -> OR. Default is AND value
				(bool)is_disable: this option create (userAccountControl:1.2.840.113556.1.4.803:=2) subsentence
				(dict)**fields: methods parameters
			
			@return: (string) ldap query sentence


			Example:
				Query = ad.setQuery(attr1="value1",attr2__not="value2",attr3__gt=["value3","value4","value5"])
				ad.objs.filter(Query)

				or

				ad.objs.filter(ad.setQuery(attr1="value1",attr2__not="value2",attr3__gt=["value3","value4","value5"])	)

				Query == '(&(!(attr2=value2))(attr3>value3)(attr3>value4)(attr3>value5)(attr1=value1))'
		'''
		qry = ''

		is_disable = fields.get('is_disable',None)

		if is_disable != None:
			if is_disable:
				qry += '(userAccountControl:1.2.840.113556.1.4.803:=2)'
			else:
				qry += '(!(userAccountControl:1.2.840.113556.1.4.803:=2))'
			del fields['is_disable']

		for i in fields:
			negative = False
			if i.find("__not") > -1:
				key = i.replace("__not","")
				negative = True
			else:
				key = i


			if key.find("__gt") > -1:
				field = key.replace("__gt","")
				optr = ">"
			elif i.find("__ge") > -1:
				field = key.replace("__ge","")
				optr = ">="
			elif i.find("__lt") > -1:
				field = key.replace("__lt","")
				optr = "<"
			elif i.find("__le") > -1:
				field = key.replace("__le","")
				optr = "<="
			else:
				field = key
				optr = "="

			grp = ''
			if type(fields[i]) == list:
				for j in fields[i]:
					filt = '(%(field)s%(optr)s%(value)s)' % {'field': field,'optr': optr, 'value': j}
					if negative:
						grp += '(!'+filt+')'
					else:
						grp += filt
						
			else:
				filt = '(%(field)s%(optr)s%(value)s)' % {'field': field,'optr': optr, 'value': fields[i]}
				if negative:
					grp = '(!'+filt+')'
				else:
					grp += filt

			qry += grp

		query = '(%(operator)s%(query)s)' % { 'operator':operator,'query': qry }
		return query



# -*- coding: utf-8 -*-

import ldap, random
from socket import gethostbyname
from urlparse import urlparse as getFQDN

from adutils import adUtils

from adobjs import adObjs, adGroups, adComputers, adUsers, adOUs

class adLDAP(adUtils):
	''' Principal Class to init the Framework	
	
		 Example:
		 	ad = adLDAP(dcs=[dc1.fqdn,dc2.fqdn,dc3.fqdn,fqdn],username="usernameAD@fqdn",password="secret")

			or

			dictconnection = {
				"dcs" : [dc1.fqdn,dc2.fqdn,dc3.fqdn,fqdn],
				"username" : "usernameAD@fqdn",
				"password" : "secret"
			}
			ad = adLDAP(**dictconnection)
	'''

	def __init__(self,dcs,username,password,basedn=None):
		'''Method construct.
			@params:
				(List)dcs : Domain Controller list.
				(String)username : username to Active Directory ldap connect
				(string)password : the username password
				(string)basedn : minimun dcs to ldap connection must be work.

			@return: None
		'''
		if type(dcs) is not list:
			raise Exception("dcs argument is not a valid list")
		self.controller = None
		self.domainControllers = dcs
		self.username = username
		self.password = password
		self.baseDN = basedn
		if not self.connect():
			raise ldap.INVALID_CREDENTIALS("Invalid Username or Password")

	def __del__(self):
		'''Method destruct.
			@params: None
			@return: None
		'''
		self.ldapConnection.unbind_s()

	def getController(self):
		'''Method to get random DC from domainControllers attribute.
			
			Select a random domain controller to replace FQDN by IP to allways connect same DC.

			@params: None
			@return: None
			@attributes:
				(List)domainControllers = List domain controllers to connect
				(str)controller = Controller selected to connect
				(str)baseDN = if not set baseDN in method construct the method set attribute from FQDN DC
				(bool)secureConnection = set attribute if ldap is secure
		'''
		if not self.controller:
			uri = random.choice(self.domainControllers)
			dc = getFQDN(uri).netloc
			ip = gethostbyname(dc)
			if not self.baseDN:
				self.baseDN = 'dc='+dc.replace('.',',dc=')
			self.controller = uri.replace(dc,ip)
			if self.controller.find('ldaps') > -1:
				self.secureConnection = True
			else:
				self.secureConnection = False
		return self.controller

	def connect(self):
		'''Method to configure minimun ldap parameters and create bind connection
			@params: none
			@return: None
			@attributes: None
		'''
		self.ldapConnection = ldap.initialize(self.getController())
		self.ldapConnection.set_option(ldap.OPT_PROTOCOL_VERSION,3)
		self.ldapConnection.set_option(ldap.OPT_REFERRALS,0)
		if self.secureConnection:
			self.ldapConnection.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_ALLOW)
		return self.authentication()

	def authentication(self):
		'''Method to create bind connection and valid user credentials'''
		try:
			self.ldapConnection.simple_bind_s(self.username,self.password)
			return True
		except ldap.INVALID_CREDENTIALS:
			return False


	def getADmaxPwdAge(self):
		'''Method to get maxPwdAge from principal DN'''
		info = self.objs.filter('(objectclass=*)',attr=['maxPwdAge'],ldapscope=ldap.SCOPE_BASE,control_type=False)[0]
		return info.maxpwdage.value

	def getDomainRID(self):
		'''Method to get Domain RID from principal DN'''
		info = self.objs.filter('(objectclass=*)',attr=['objectsid'],ldapscope=ldap.SCOPE_BASE,control_type=False)[0]
		return info.objectsid.value

	@property
	def users(self):
		'''Method like property to init adUser class
			
			Example:
				ad.users.new()

				or

				ad.users.get('(attribute=value)')
		'''
		return adUsers(self)

	@property
	def computers(self):
		'''Method like property to init adComputers class

			Example:
				ad.computers.new()

				or

				ad.computers.get('(attribute=value)')
		'''
		return adComputers(self)
	
	@property
	def groups(self):
		'''Method like property to init adGroups class
			
			Example:
				ad.groups.new()

				or

				ad.groups.get('(attribute=value)')
		'''
		return adGroups(self)

	@property
	def objs(self):
		'''Method like property to init adObjs class

			Example:
				ad.objs.new()

				or

				ad.objs.get('(attribute=value)')
		'''
		return adObjs(self)

	@property
	def ous(self):
		'''Method like property to init adOUs class

			Example:
				ad.ous.new()

				or

				ad.ous.get('(attribute=value)')
		'''
		return adOUs(self)



# -*- coding: utf-8 -*-
import json, uuid, datetime
from adutils import adUtils


class adGenericAttr(object):
	def __init__(self,adldap,value):
		self.adldap = adldap
		if type(value) != list:
			raise Exception("Value must be list")
		self.ValueToRaw(value)

	def __getitem__(self,item):
		return self.raw[item]

	def __setitem__(self,item,value):
		self.raw[item] = value
		self.is_modified = True

	def __delitem__(self,item):
		del self.raw[item]
		self.is_modified = True

	@property
	def value(self):
		return self.RawToValue()

	def ValueToRaw(self,value):
		self.raw = value

	def RawToValue(self):
		if len(self.raw) > 1:
			return self.raw
		else:
			if len(self.raw) == 0:
				return None
			else:
				return self.raw[0]

	def humanReadeable(self):
		return self.value

	def __repr__(self):
		return repr(self.value)

	def __str__(self):
		return str(self.humanReadeable())

class PrimaryGroupID(adGenericAttr):
	def humanReadeable(self):
		 gsid = self.adldap.getDomainRID()+'-'+self.raw[0]
		 return self.adldap.groups.get(self.adldap.setQuery(objectsid=gsid)).dn.value


class unicodePwd(adGenericAttr):
	def ValueToRaw(self,password):
		unicode_pass = unicode('\"' + str(password[0]) + '\"', 'iso-8859-1')
		super(unicodePwd,self).ValueToRaw([unicode_pass.encode("utf-16-le")])

class UserAccountControl(adGenericAttr):
	def ValueToRaw(self,value):
		if type(value) == list:
			acControl = self.adldap.accountControl(value)
			if acControl > 0:
				self.raw = [str(self.adldap.accountControl(value))]
			else:
				self.raw = value
		else:
			self.raw = [str(value)]

	def humanReadeable(self):
		if self.value == None:
			return str(self.value)
		binary = bin(int(self.value))[2:][::-1]
		values = []
		for bit in range(0,len(binary)):
			if binary[bit] == '1':
				decimal = int(binary[bit]+'0'*bit,2)
				try:
					position = self.adldap.acControl.values().index(decimal)
					value = self.adldap.acControl.keys()[position]
					values.append(value)
				except:
					values.append("UNKNOWN")
		return values


class dn(adGenericAttr):
	pass


class adTime0Z(adGenericAttr):	
	def humanReadeable(self):
		val = []
		for d in self.raw:
			human = "%(year)s-%(month)s-%(day)s %(hour)s:%(min)s:%(sec)s" % { "year":d[0:4],"month":d[4:6],"day":d[6:8],
																									"hour":d[8:10],"min": d[10:12],"sec": d[12:14]}
			val.append(human)
		return val


class WhenChanged(adTime0Z):
	pass


class WhenCreated(adTime0Z):
	pass


class dSCorePropagationData(adTime0Z):
	pass


class adWinTimeStamp(adGenericAttr):	
	def Win2UnixTimeStamp(self,wints):
		if (wints == 0) or (int(wints) == 9223372036854775807):
			return 0

		secsAfterADEpoch = int(wints) / float(10000000)
		AD2Unix = ((1970-1601) * 365 - 3 + (1970-1601)/4) * 86400
		return int(secsAfterADEpoch - AD2Unix)

	def humanReadeable(self):
		val = []
		for ts in self.raw:
			unixTimeStamp = self.Win2UnixTimeStamp(ts)
			if unixTimeStamp <= 0:
				val.append(0)
			else:
				d = datetime.datetime.fromtimestamp(unixTimeStamp)
				val.append(d.strftime("%Y-%m-%d %H:%M:%S"))
		return val


class badPasswordTime(adWinTimeStamp):
	pass


class lastLogon(adWinTimeStamp):
	pass


class pwdLastSet(adWinTimeStamp):
	pass


class lastLogonTimeStamp(adWinTimeStamp):
	pass


class accountExpires(adWinTimeStamp):
	pass


class mSMQSignCertificates(adGenericAttr):
	pass


class adGenericMember(adGenericAttr):
	def __getitem__(self,index):
		return self.adldap.objs.get(self.adldap.setQuery(distinguishedname=self.raw[index]))

	def __setitem__(self,index,item):
		pass

	def is_member(self,groupname):
		for i in self.value:
			grp = i.lower().split(',')[0].replace('cn=','')
			if grp.strip() == groupname.lower().strip():
				return True
		return False
			
class memberOf(adGenericMember):
	pass

class member(adGenericMember):
	pass


class sAMAccountName(adGenericAttr):
	pass


class ObjectGUID(adGenericAttr):
	def RawToValue(self):
		return str(uuid.UUID(bytes=self.raw[0]))


class adGenericSid(adGenericAttr):
	def RawToValue(self):
		return adUtils.getTextSID(self.raw[0])

class ObjectSid(adGenericSid):
	pass

class sIDHistory(adGenericSid):
	pass

class userWorkStations(adGenericAttr):
	pass

class logonHours(adGenericAttr):
	def __getitem__(self,item):
		return self.humanReadeable()[item]

	def RawToValue(self):
		return list(bytearray(self.raw[0]))

	def humanReadeable(self):
		value = self.value
		sun = bin(value[0])[2:].zfill(8)[::-1]+bin(value[1])[2:].zfill(8)[::-1]+bin(value[2])[2:].zfill(8)[::-1]
		mon = bin(value[3])[2:].zfill(8)[::-1]+bin(value[4])[2:].zfill(8)[::-1]+bin(value[5])[2:].zfill(8)[::-1]
		tue = bin(value[6])[2:].zfill(8)[::-1]+bin(value[7])[2:].zfill(8)[::-1]+bin(value[8])[2:].zfill(8)[::-1]
		wed = bin(value[9])[2:].zfill(8)[::-1]+bin(value[10])[2:].zfill(8)[::-1]+bin(value[11])[2:].zfill(8)[::-1]
		thu = bin(value[12])[2:].zfill(8)[::-1]+bin(value[13])[2:].zfill(8)[::-1]+bin(value[14])[2:].zfill(8)[::-1]
		fri = bin(value[15])[2:].zfill(8)[::-1]+bin(value[16])[2:].zfill(8)[::-1]+bin(value[17])[2:].zfill(8)[::-1]
		sat = bin(value[18])[2:].zfill(8)[::-1]+bin(value[19])[2:].zfill(8)[::-1]+bin(value[20])[2:].zfill(8)[::-1]

		week = {
			"sun" : sun,
			"mon" : mon,
			"tue" : tue,
			"wed" : wed,
			"thu" : thu,
			"fri" : fri,
			"sat" : sat,
		}
		return week


class manager(adGenericAttr):
	pass

class directreports(adGenericAttr):
	pass



from distutils.core import setup

requires = []

try:
	import ldap
except:
	requires.append('python-ldap')

setup(
	name="pyadldap",
	version="0.1.3",
	description="FrameWork for managing Microsoft Active Directory from LDAP",
	author="Jonas Delgado Mesa",
	author_email="jdelgado@yohnah.net",
	url="https://github.com/Wengex/PyADLdap",
	license="GPLv2",
	packages=["pyadldap"],
	long_description=open('README.md').read(),
	install_requires = requires
)
